-- CreateTable
CREATE TABLE "User" (
    "id" INTEGER NOT NULL,
    "name" TEXT,
    "mail" TEXT,
    "gitlabUsername" TEXT NOT NULL,
    "deleted" BOOLEAN NOT NULL DEFAULT false,

    CONSTRAINT "User_pkey" PRIMARY KEY ("id")
);

-- CreateTable
CREATE TABLE "QcmTable" (
    "idQCM" SERIAL NOT NULL,
    "nomQCM" TEXT NOT NULL,
    "temps" INTEGER NOT NULL,
    "randomOrder" BOOLEAN NOT NULL,
    "codeAcces" INTEGER NOT NULL,
    "idUserCreator" INTEGER NOT NULL,

    CONSTRAINT "QcmTable_pkey" PRIMARY KEY ("idQCM")
);

-- CreateTable
CREATE TABLE "Type" (
    "idType" SERIAL NOT NULL,
    "nomType" TEXT NOT NULL,

    CONSTRAINT "Type_pkey" PRIMARY KEY ("idType")
);

-- CreateTable
CREATE TABLE "Choix" (
    "idChoix" SERIAL NOT NULL,
    "isCorrect" BOOLEAN NOT NULL,
    "idQuestion" INTEGER NOT NULL,
    "nomChoix" TEXT NOT NULL,
    "position" INTEGER NOT NULL,

    CONSTRAINT "Choix_pkey" PRIMARY KEY ("idChoix")
);

-- CreateTable
CREATE TABLE "Question" (
    "idQuestion" SERIAL NOT NULL,
    "question" TEXT NOT NULL,
    "position" INTEGER NOT NULL,
    "randomOrder" BOOLEAN NOT NULL,
    "nbPtsPositif" INTEGER NOT NULL,
    "nbPtsNegatif" INTEGER NOT NULL,
    "numeric" DOUBLE PRECISION,
    "idQCM" INTEGER NOT NULL,
    "idType" INTEGER NOT NULL,
    "isMultiple" BOOLEAN NOT NULL,

    CONSTRAINT "Question_pkey" PRIMARY KEY ("idQuestion")
);

-- CreateTable
CREATE TABLE "Reponse" (
    "idReponse" SERIAL NOT NULL,
    "idQuestion" INTEGER NOT NULL,
    "idChoix" INTEGER,
    "idUser" INTEGER NOT NULL,
    "numeric" DOUBLE PRECISION,

    CONSTRAINT "Reponse_pkey" PRIMARY KEY ("idReponse")
);

-- CreateTable
CREATE TABLE "Participer" (
    "idUser" INTEGER NOT NULL,
    "idQCM" INTEGER NOT NULL,
    "feedback" TEXT NOT NULL,
    "note" DOUBLE PRECISION NOT NULL,
    "score" INTEGER NOT NULL,
    "heureDebut" INTEGER NOT NULL,
    "hasFinished" BOOLEAN NOT NULL DEFAULT false,

    CONSTRAINT "Participer_pkey" PRIMARY KEY ("idUser","idQCM")
);

-- CreateIndex
CREATE UNIQUE INDEX "User_mail_key" ON "User"("mail");

-- CreateIndex
CREATE UNIQUE INDEX "User_gitlabUsername_key" ON "User"("gitlabUsername");

-- CreateIndex
CREATE UNIQUE INDEX "Type_nomType_key" ON "Type"("nomType");

-- CreateIndex
CREATE UNIQUE INDEX "Reponse_idQuestion_idUser_idChoix_key" ON "Reponse"("idQuestion", "idUser", "idChoix");

-- AddForeignKey
ALTER TABLE "QcmTable" ADD CONSTRAINT "QcmTable_idUserCreator_fkey" FOREIGN KEY ("idUserCreator") REFERENCES "User"("id") ON DELETE RESTRICT ON UPDATE CASCADE;

-- AddForeignKey
ALTER TABLE "Choix" ADD CONSTRAINT "Choix_idQuestion_fkey" FOREIGN KEY ("idQuestion") REFERENCES "Question"("idQuestion") ON DELETE CASCADE ON UPDATE CASCADE;

-- AddForeignKey
ALTER TABLE "Question" ADD CONSTRAINT "Question_idQCM_fkey" FOREIGN KEY ("idQCM") REFERENCES "QcmTable"("idQCM") ON DELETE RESTRICT ON UPDATE CASCADE;

-- AddForeignKey
ALTER TABLE "Question" ADD CONSTRAINT "Question_idType_fkey" FOREIGN KEY ("idType") REFERENCES "Type"("idType") ON DELETE RESTRICT ON UPDATE CASCADE;

-- AddForeignKey
ALTER TABLE "Reponse" ADD CONSTRAINT "Reponse_idQuestion_fkey" FOREIGN KEY ("idQuestion") REFERENCES "Question"("idQuestion") ON DELETE RESTRICT ON UPDATE CASCADE;

-- AddForeignKey
ALTER TABLE "Reponse" ADD CONSTRAINT "Reponse_idChoix_fkey" FOREIGN KEY ("idChoix") REFERENCES "Choix"("idChoix") ON DELETE CASCADE ON UPDATE CASCADE;

-- AddForeignKey
ALTER TABLE "Reponse" ADD CONSTRAINT "Reponse_idUser_fkey" FOREIGN KEY ("idUser") REFERENCES "User"("id") ON DELETE RESTRICT ON UPDATE CASCADE;

-- AddForeignKey
ALTER TABLE "Participer" ADD CONSTRAINT "Participer_idUser_fkey" FOREIGN KEY ("idUser") REFERENCES "User"("id") ON DELETE RESTRICT ON UPDATE CASCADE;

-- AddForeignKey
ALTER TABLE "Participer" ADD CONSTRAINT "Participer_idQCM_fkey" FOREIGN KEY ("idQCM") REFERENCES "QcmTable"("idQCM") ON DELETE RESTRICT ON UPDATE CASCADE;
