import process from 'process';
import logger from '../src/logging/WinstonLogger.js';
import db from '../src/helpers/DatabaseHelper.js';
import Config from '../src/config/Config.js';

async function main() {
    await users();
    await types();
}

main().then(async () => {
    await db.$disconnect();
}).catch(async e => {
    logger.error(JSON.stringify(e));
    await db.$disconnect();
    process.exit(1);
});

//----------------------------------------------------------------------------------------------------------------------------------------------------------

async function users() {
    await db.user.upsert({
        where: { id: 142 },
        update: {
            name: 'Michaël Minelli'
        },
        create: {
            id: 142,
            name: 'Michaël Minelli',
            gitlabUsername: 'michael.minelli',
            deleted: false
        }
    });

    if (!Config.production) {
        await db.user.upsert({
            where: { id: 525 },
            update: {
                deleted: false
            },
            create: {
                id: 525,
                gitlabUsername: 'stephane.malandai',
                deleted: false
            }
        });
    }
}

async function types() {
    await db.type.upsert({
        where: { nomType: 'text' },
        update: {},
        create: {
            nomType: 'text'
        }
    });

    await db.type.upsert({
        where: { nomType: 'numerique' },
        update: {},
        create: {
            nomType: 'numerique'
        }
    });

    await db.type.upsert({
        where: { nomType: 'vraiFaux' },
        update: {},
        create: {
            nomType: 'vraiFaux'
        }
    });

    console.log("✅ Types de base insérés ou déjà présents.");
}
