import express         from 'express';

import db               from '../helpers/DatabaseHelper.js';
import { verifyJWT } from '../Middlewares.js';
import { checkUser } from "../calcFunctions.js"
import { MessageRoute } from './MessageRoute.js';

import axios from 'axios';
import jwt from 'jsonwebtoken';

const router: express.Router = express.Router();
const redirectUri = 'http://localhost:4200';
router.post('/auth/jwt', async (req, res) => {
    const { code } = req.body;
    if (!code) {
        res.status(400).send('Code is required');
    }
    try {
        //Demande access_token user avec le code
        const response = await axios.post('https://githepia.hesge.ch/oauth/token', {
            client_id: String(process.env.CLIENTID),
            client_secret: String(process.env.CLIENTSECRET),
            code: code,
            grant_type: 'authorization_code',
            redirect_uri: redirectUri,
        });
        
        const { access_token } = response.data;
        
        //Demande du name et de l'email utilisateur
        const userResponse = await axios.get('https://githepia.hesge.ch/api/v4/user', {
            headers: { Authorization: `Bearer ${access_token}` }
        });

        const { id, name, email } = userResponse.data;
        console.log(id, name, email)
        if(name || email || id){
            //erreur
        }
        const infoQcm = await db.user.findFirst({
            where: {
                name: name,
                mail: email
            }
        });
        // Génération d'un token JWT personnalisé
        if(!infoQcm){ 
            const createUser = await db.user.create({
                data: {
                    id: id,
                    gitlabUsername: name,
                    mail: email,
                    deleted: false,
                    name: name,
                }
            }); 
            if(!createUser){
                res.status(500).send({error: 'Error create user'});
            }
        }
        const jwtToken = jwt.sign({ id }, String(process.env.SECRET_JWT), {expiresIn: '1h'});

        res.json({ 
            token: jwtToken,
            idUser: id
        });

    } catch (error) {
        res.status(500).send('Error exchanging code for token');
    }
});


export default router;
