
import db               from './helpers/DatabaseHelper.js';
import { randomInt } from 'crypto';
import express         from 'express';

export function calcTempsRestant(heureDebut: number, tempsQCM: number) : number
{
    const deltaTemps = Math.floor(Date.now() / 1000) - heureDebut;
    const tempsRestant = tempsQCM - deltaTemps;
    return tempsRestant > 0 ? tempsRestant : -1;
}

export async function calcNbPtsTotalQCM(idQCM: number) : Promise<number>
{
    const questions = await db.question.findMany({
        where: {
            idQCM: idQCM
        },
        select: {
            nbPtsPositif: true
        }
    });
    return questions.reduce((total, question) =>  total + (question.nbPtsPositif || 0), 0);
}

export function calcNoteBaremeFed(nbPtsObtenus: number, nbPtsTotal: number) : number
{
    return nbPtsObtenus / nbPtsTotal * 5 + 1;
}

export function getRandomNumber(min: number, max: number): number {
    return randomInt(min, max + 1); // `randomInt` génère un nombre entier aléatoire entre `min` (inclus) et `max` (exclus).
}

export function checkUser(req: express.Request, res: express.Response, userId: number, errorMessage: string)
{
    if (req.user) {
        const { id } = req.user;
        if (id !== userId)
        {
            res.status(403).send(errorMessage);
        }
    } else {
        res.status(401).send('Unauthorized');
    }
    
}

