import express         from 'express';
import db               from '../helpers/DatabaseHelper.js';
import reqGetDB           from './reqGetDB.js';
import { verifyJWT } from '../Middlewares.js';
import { checkUser, calcNbPtsTotalQCM } from "../calcFunctions.js"
import { MessageRoute } from './MessageRoute.js';


const router: express.Router = express.Router();
async function getResponsesQCM(QCM_ID: number, USER_ID: number)
{
    return db.qcmTable.findUnique({
        where: { idQCM: QCM_ID },
        select: {
            questions: {
                select: {
                    idQuestion: true,
                    reponses: {
                        where: { idUser: USER_ID },
                        select: {
                            idChoix: true,
                            idReponse: true,
                            numeric: true
                        }
                    }
                }
            }
        }
    });
}

router.get('/reponseCorrect/:QCM_ID', async (req: express.Request, res: express.Response) => {
    const QCM_ID: number = parseInt(req.params.QCM_ID, 10);
    const questions = await db.question.findMany({
        where: {
            idQCM: QCM_ID
        },
        include: {
            choix: true
        }
    });
    if(!questions){
        res.status(500).send({ error: MessageRoute.qcmDoesntExist });
    }
    const reponsesCorrect: number[][] = [];
    for (const question of questions) {
        const reponse: number[] = [];
        const correctChoices = question.choix.filter(choice => choice.isCorrect);
        if (question.numeric) {
            reponse.push(question.numeric);
        } else {
            correctChoices.forEach(choice => {
                reponse.push(choice.idChoix);
            });
        }
        reponsesCorrect.push(reponse);
    }
    res.status(200).send(reponsesCorrect);
});

router.get('/responses_QCM/:QCM_ID/:USER_ID', verifyJWT, async (req: express.Request, res: express.Response) => {
    const QCM_ID: number = parseInt(req.params.QCM_ID, 10);
    const USER_ID: number = parseInt(req.params.USER_ID, 10);
    const results = await reqGetDB(() => getResponsesQCM(QCM_ID, USER_ID));
    res.send(results);
});

router.put('/feedback', verifyJWT, async (req: express.Request, res: express.Response) => {
    const { idQCM, idUser, note, feedback } = req.body;
    console.log(idQCM, idUser, note, feedback)
    if (!idQCM || note === undefined || feedback === undefined || idUser === undefined)
    {
        res.status(400).send({ error:  MessageRoute.misPara });
        return
    }
    const infoQcm = await db.participer.findFirst({
        where: {
            AND: {
                idQCM: idQCM,
                idUser: idUser,
            }
        },
        include: {
            qcm: true
        }
    });
    if(!infoQcm)
    {
        res.status(404).send({ error: MessageRoute.qcmDoesntExist });
        return
    }
    checkUser(req, res, infoQcm.qcm.idUserCreator, "You can't access this ressource");
    const updateFeddback = await db.participer.update({
        where: {
            idUser_idQCM: {
                idQCM: idQCM,
                idUser: idUser
            }

        },
        data: {
            feedback: feedback,
            note: note,
        },
    });
    if(!updateFeddback){
        res.status(500).send({ error: 'Error FeedBack' });
        return
    }
    res.status(200).send();
});

router.get('/results/:QCM_ID', verifyJWT, async (req: express.Request, res: express.Response) => {
    const QCM_ID: number = parseInt(req.params.QCM_ID, 10);

    // Fetch QCM details
    const qcm = await db.qcmTable.findUnique({
        where: {
            idQCM: QCM_ID,
        },
        include: {
            participer: {
                include: {
                    user: true,
                },
            },
            questions:true
        },
    });

    let moy = 1;
    let cmp = 0;
    if(qcm?.participer){
        qcm.participer.forEach(part =>  cmp += part.note)      
        moy =  cmp /  qcm.participer.length;
    }
    const nbPoint: number = await calcNbPtsTotalQCM(QCM_ID);
    res.send({
        qcm : qcm,
        moyClasse : moy,
        scoreMax : nbPoint,
    });
});
export default router;
