import express         from 'express';
import db               from '../helpers/DatabaseHelper.js';
import reqGetDB           from './reqGetDB.js';
import { calcNbPtsTotalQCM, getRandomNumber, checkUser } from "../calcFunctions.js"
import { verifyJWT } from '../Middlewares.js';
import { randomInt } from 'crypto';
import { MessageRoute } from './MessageRoute.js';

const router: express.Router = express.Router();
// router.use(verifyJWT)

router.post('/QCM', verifyJWT, async (req: express.Request, res: express.Response) => {
    console.log(`[LOG] Ajout QCM avec les données:`, req.body);
    const { qcm, idUser } = req.body;
    console.log(qcm, idUser, qcm["nomQcm"], qcm["randomQuestion"], qcm["tempsMax"])
    if (!qcm || !idUser || !qcm["nomQcm"] || qcm["randomQuestion"] === undefined || qcm["tempsMax"] === undefined)
    {
        res.status(400).send({ error:  MessageRoute.misPara });
        return
    }
    const codeAcces: number = getRandomNumber(1000, 9999);
    const qcmCreate = await db.qcmTable.create({
        data: {
            nomQCM :  qcm["nomQcm"],
            codeAcces : codeAcces,
            randomOrder : qcm["randomQuestion"],
            temps : qcm["tempsMax"],
            idUserCreator : idUser,
        }
    });
    res.status(200).send({id: qcmCreate.idQCM});
});

router.put('/QCM', verifyJWT, async (req: express.Request, res: express.Response) => {
    console.log(`[LOG] Traitement de la création de QCM avec les données:`, req.body);
    const { qcm } = req.body;
    console.table(qcm)
    if (!qcm || !qcm["nomQcm"] || qcm["randomQuestion"] === undefined || qcm["tempsMax"] === undefined || qcm["idQcm"] === undefined)
    {
        res.status(400).send({ error:  MessageRoute.misPara });
        return
    }
    const infoQcm = await db.qcmTable.findFirst({
        where: {
            idQCM: qcm["idQcm"],
        }
    });
    
    if(!infoQcm)
    {
        res.status(404).send({ error: MessageRoute.qcmDoesntExist });
        return
    }
    checkUser(req, res, infoQcm?.idUserCreator, "This is not your QCM");
    const qcmCreate = await db.qcmTable.update({
        where: {
            idQCM: qcm["idQcm"]
        },
        data: {
            nomQCM :  qcm["nomQcm"],
            randomOrder : qcm["randomQuestion"],
            temps : qcm["tempsMax"],
        },
    });
    res.status(200).send(qcmCreate);
});


router.post('/numeric_question', verifyJWT, async (req: express.Request, res: express.Response) => {
    console.log(`[LOG] Ajout de question numérique avec les données:`, req.body);
    const { question, idQcm } = req.body;
   
    console.log(question, idQcm);
    if (!idQcm || !question || !question["question"] || question["nbPtsNegatif"] === undefined || question["nbPtsPositif"] === undefined || question["valNum"] === undefined || question["position"] === undefined)
    {
        res.status(400).send({ error:  MessageRoute.misPara });
        return
    }
    const infoQcm = await db.qcmTable.findFirst({
        where: {
            idQCM: idQcm
        }
    });
    if(!infoQcm)
    {
        res.status(404).send({ error: MessageRoute.qcmDoesntExist });
        return
    }
    checkUser(req, res, infoQcm.idUserCreator, MessageRoute.cantCreate);
    const type = await db.type.findFirst({
        where: {
            nomType: "numerique"
        },
    });
    if(!type)
    {
        res.status(500).send({ error: 'Server error' });
        return
    }
    const qcmCreate = await db.question.create({
        data: {
            nbPtsNegatif: question["nbPtsNegatif"],
            nbPtsPositif: question["nbPtsPositif"],
            position: question["position"],
            isMultiple: false,
            question: question["question"],
            idQCM: idQcm,
            idType: type["idType"],
            numeric: question["valNum"],
            randomOrder: false,
        }
    });
    res.status(200).send({id: qcmCreate.idQuestion});
});

router.post('/true_false_question', verifyJWT, async (req: express.Request, res: express.Response) => {
    console.log(`[LOG] Ajout question true_false avec les données:`, req.body);
    try {
        const { question, idQcm } = req.body;
        console.table(req.body)
        console.log(question, idQcm)
        if (!idQcm || !question || !question["question"] || !question["nbPtsNegatif"] === undefined || !question["nbPtsPositif"] === undefined || question["isVraiFaux"] === undefined || question["valVraiFaux"] === undefined) {
            res.status(400).send({ error:  MessageRoute.misPara });
            return
        }
    
        const infoQcm = await db.qcmTable.findFirst({
          where: {
            idQCM: idQcm
          }
        });
    
        if (!infoQcm) {
            res.status(400).send({ error:MessageRoute.qcmDoesntExist});
            return
        }
        checkUser(req, res, infoQcm.idUserCreator, MessageRoute.cantCreate);
        
        const type = await db.type.findFirst({
          where: {
            nomType: "vraiFaux"
          }
        });
    
        if (!type) {
            res.status(500).send({ error: 'Server Problem: Type not found' });
            return
        }
    
    
        const questionCreate = await db.question.create({
          data: {
            nbPtsNegatif: question["nbPtsNegatif"],
            nbPtsPositif: question["nbPtsPositif"],
            position: 0,
            isMultiple: false,
            question: question["question"],
            idQCM: idQcm,
            idType: type["idType"],
            randomOrder: question["randomOrder"],
          }
        });
    
    
        if (!questionCreate) {
            res.status(500).send({ error: 'Server Problem: Question creation failed' });
            return
        }
    
        const idQuestion = questionCreate.idQuestion;
    
        const choixData = [
          { nomChoix: "Vrai", isCorrect: question.valVraiFaux, idQuestion: idQuestion, position: 0 },
          { nomChoix: "Faux", isCorrect: !question.valVraiFaux, idQuestion: idQuestion, position: 1 }
        ];
    
        const choixCreate = await db.choix.createMany({
          data: choixData
        });
        if(!choixCreate){
            res.status(500).send({ error:  MessageRoute.serverError });
            return
        }
    
        res.status(200).send({id:  idQuestion});
    } catch (error) {
        res.status(500).send({ error:  MessageRoute.serverError });
    }
});

export default router;
