import { Component, inject } from '@angular/core';
import { QcmData } from '../../classes/qcm/QcmData';
import { Fetch } from '../../classes/Fetch';
import { AuthConfigService } from '../../auth-config.service';
import { ActivatedRoute, Router } from '@angular/router';
import { timer, Observable } from 'rxjs';
import { map, tap, takeWhile } from 'rxjs/operators';

@Component({
  selector: 'app-examen',
  templateUrl: './examen.component.html',
  styleUrl: './examen.component.scss'
})
export class ExamenComponent {
  auth: AuthConfigService;
  questionActuel: number = 0
  qcm!: QcmData
  idQcm: number
  tempsRestant?: Observable<number>;
  maxTime: number;

  constructor(private readonly router: Router, private readonly route: ActivatedRoute) {
    this.auth = inject(AuthConfigService)
    this.auth.redirectNoLogin()
    const idParam = this.route.snapshot.paramMap.get('id');
    if (idParam !== null) {
      this.idQcm = +idParam;
    } else {
      this.idQcm = 0;
      console.error('ID param is null');
    }
    this.maxTime = 0;
    Fetch.getQcm(this.idQcm)
    .then(data => {
      this.maxTime = data.tempsMax;
      Fetch.getReponses(this.idQcm, Number(localStorage.getItem('idUser')), data).then(val => {
        this.qcm = val
        this.timerStart()
      });
    })
    .catch(error => console.error('Erreur lors de la requête pour Contact:', error))
  }
  next(): void{
    if(this.questionActuel !== this.qcm.questions.length - 1){
      this.questionActuel += 1;
    }
  }
  terminerTest(): void{
    Fetch.putTerminer(Number(localStorage.getItem('idUser')), this.idQcm).then(() => this.router.navigate(['/qcmRealise']))
  }
  previous(): void{
    if(this.questionActuel !== 0){
      this.questionActuel -= 1;
    }
  }
  getNom(): string{
    if(this.qcm){
      return this.qcm.nomQcm
    }
    return ""
  }
  timerStart(): void {
    const currentTimeSeconds = Math.floor(Date.now() / 1000);
    console.log(this.qcm)
    const startTimeSeconds = this.qcm.tempsStart;
    
    const totalRemainingTime = startTimeSeconds + this.qcm.tempsMax - currentTimeSeconds;
    
    this.tempsRestant = timer(0, 1000).pipe(
      map(n => (totalRemainingTime - n) * 1000),
      tap(n => {
        if (n <= 0) {
          this.finishAttempt();
        }
      }),
      takeWhile(n => n >= 0)
    );
  }
  finishAttempt() {
    this.terminerTest()
  }
}
