import { FeedBack } from "./FeedBack";
import { ParticipantQcm } from "./resultat/ParticpantQcm";
import { QcmCree } from "./creer/QcmCree";
import { QcmRealise } from "./realiser/QcmRealise";
import { ResultatQcm } from "./resultat/ResultatQcm";
import { QcmData } from "./qcm/QcmData";
import { Question } from "./qcm/Question";
import { Choix } from "./qcm/Choix";
import { Reponse } from "./qcm/Reponse";
import { QuestionRes } from "./resultat/QuestionRes";

type HttpMethod = 'GET' | 'POST' | 'PUT' | 'DELETE' | 'PATCH';

interface FetchOptions {
  method: HttpMethod;
  address: string;
  route: string;
}

export class Fetch{
    static address: string = "http://0.0.0.0:30992";
    static messageErrorFormat: string = "Invalid response format"
    static headersObject: HeadersInit = {
        'Content-Type': 'application/json',
        'Authorization': 'Bearer '+localStorage.getItem('access_token') ,
    };

    static requeteServeur(Fo : FetchOptions): Promise<Response>{
        return new Promise<Response>((resolve, reject) => {
            fetch(`${Fo.address}/${Fo.route}`, {
                method: Fo.method,
                headers: this.headersObject,
            })
            .then(rep => resolve(rep))
            .catch(error => reject(error));
        });
    }
    static getReponsesCorrectQcm(QCM_ID: number): Promise<number[][]> {
        return new Promise<number[][]>((resolve, reject) => {
            this.requeteServeur({
                method: 'GET',
                address: Fetch.address,
                route: `reponseCorrect/${QCM_ID}`,
            })
            .then(data => data.json())
            .then(data => {
                if (!Array.isArray(data)) {
                    reject(new Error(Fetch.messageErrorFormat));
                    return;
                }
                console.log("rea", data)
                resolve(data);
            })
            .catch(error => reject(error));
        })
    }

    // ----- Get -----
    static getQcmsRealise(USER_ID: number): Promise<QcmRealise[]> {
        console.log(localStorage.getItem('access_token'))
        return new Promise<QcmRealise[]>((resolve, reject) => {
            this.requeteServeur({
                method: 'GET',
                address: Fetch.address,
                route: `realised_QCMs/${USER_ID}`,
            })
            .then(data => data.json())
            .then(data => {
                console.log(data);
                if (!Array.isArray(data)) {
                    reject(new Error(Fetch.messageErrorFormat));
                    return;
                }
                const qcms: QcmRealise[] = [];
                for (let i = 0; i < data.length; i++) {
                    const questions: QuestionRes[] = [];
                    for (const question of data[i]["qcm"]["questions"]) {
                        questions.push(new QuestionRes(question["idQuestion"]));
                    }
                    qcms[i] = new QcmRealise(data[i]["idQCM"], data[i]["qcm"]["nomQCM"], new FeedBack(data[i]["feedback"], data[i]["note"]), questions, data[i]["score"], data[i]["scoreMax"]);
                }
                resolve(qcms);
            })
            .catch(error => reject(error));
        })
    }
    static getQcmsCreer(USER_ID: number) : Promise<QcmCree[]>{   
        return new Promise<QcmCree[]>((resolve, reject) => {
            this.requeteServeur({
                method: 'GET',
                address: Fetch.address,
                route: `created_QCMs/${USER_ID}`,
            })
            .then(data => { return data.json();})
            .then(data => {
                console.log(data);
                if (!Array.isArray(data)) {
                    reject(new Error(Fetch.messageErrorFormat));
                    return;
                }
                const qcms : QcmCree[] = []
                for (let i = 0; i < data.length; i++) {
                    qcms[i] = new QcmCree(data[i]["nomQCM"],data[i]["idQCM"], data[i]["codeAcces"], data[i]["nbParticipants"] ?? 0);
                }
                resolve(qcms)
            })
            .catch(error => reject(error));
        })
    }
    static getResultatQcm(QCM_ID: number) : Promise<ResultatQcm>{
        const participants: ParticipantQcm[] = [];
        return new Promise ((resolve, reject) => {
            this.requeteServeur({
                    method: 'GET',
                    address: Fetch.address,
                    route: `results/${QCM_ID}`,
                })
                .then(data => data.json())
                .then(rep => {
                    for (const participant of rep["qcm"]["participer"]) {
                        participants.push(new ParticipantQcm(
                            participant["user"]["name"],
                            participant["user"]["name"],
                            rep["qcm"]["nomQcm"],
                            rep["scoreMax"],
                            participant["score"] ?? 0,
                            new FeedBack(participant["feedback"], participant["note"]),
                            [],
                            participant["user"]["id"]
                        ));
                    }
                    const questions: QuestionRes[] = [];
                    for (const question of rep["qcm"]["questions"]) {
                        questions.push(new QuestionRes(question["idQuestion"]));
                    }
                    resolve(new ResultatQcm(
                        rep["moyClasse"],
                        rep["qcm"]["nomQCM"],
                        participants.length,
                        participants,
                        questions,
                        rep["qcm"]["idQCM"],
                        rep["scoreMax"]
                    ));                })
                .catch(error => reject(error));
        });
    }
    static getReponsesUser(QCM_ID: number, USER_ID: number): Promise<{[key : number] : number[]}>{
        return new Promise<{[key : number] : number[]} >((resolve, reject) => {
            this.requeteServeur({
                method: 'GET',
                address: Fetch.address,
                route: `responses_QCM/${QCM_ID}/${USER_ID}`,
            })
            .then(data => data.json())
            .then(data => {
                const resAll: {[key : number] : number[]} = {}
                for (const question of data["questions"]) {
                    resAll[question["idQuestion"]] = [];
                    for (const reponse of question["reponses"]) {
                        if (reponse["numeric"]) {
                            resAll[question["idQuestion"]].push(reponse["numeric"]);
                        } else {
                            resAll[question["idQuestion"]].push(reponse["idChoix"]);
                        }
                    }
                }
                resolve(resAll)
            })
            .catch(error => reject(error));
        })
    }
    static getReponses(QCM_ID: number,USER_ID:number, qcm: QcmData): Promise<QcmData>{
        return new Promise<QcmData>((resolve, reject) => {
            this.requeteServeur({
                method: 'GET',
                address: Fetch.address,
                route: `responses_QCM/${QCM_ID}/${USER_ID}`,
            })
            .then(data => data.json())
            .then(data => {
                for (const dataQuestion of data["questions"]) {
                    const questionId = dataQuestion["idQuestion"];
                    const question: Question | undefined = qcm.questions.find(q => q.idQuestion === questionId);  
                    if (question) {
                        if (!question.reponses) {
                            question.reponses = [];
                        }
                        for (const dataReponse of dataQuestion["reponses"]) {
                            if (question.isNumerique) {
                                question.reponses.push(new Reponse(dataReponse["numeric"], dataReponse["idReponse"]));
                                question.valNum = dataReponse["numeric"];
                            } else {
                                question.reponses.push(new Reponse(dataReponse["idChoix"], dataReponse["idReponse"]));
                            }
                        }
                    }
                }
                
                resolve(qcm)
            })
            .catch(error => reject(error));
        })
    }
    static getInfosQcm(QCM_ID: number) : Promise<QcmData>{
        return new Promise ((resolve, reject) => {
            this.requeteServeur({
                method: 'GET',
                address: Fetch.address,
                route: `infos_QCM/${QCM_ID}`,
            })
            .then(response => response.json())
            .then(data => {
                const questions: Question[] = []
                for (const [i, questionData] of data["questions"].entries()) {
                    const image: string = "";
                    if (["image", "text", "vraiFaux"].includes(questionData["type"]["nomType"])) {
                        const choix: Choix[] = [];
                        // Textes
                        if (["text", "vraiFaux"].includes(questionData["type"]["nomType"])) {
                            for (const [j, choixData] of questionData["choix"].entries()) {
                                choix[j] = new Choix(choixData["idChoix"], choixData["nomChoix"], choixData["isCorrect"]);
                            }
                        }
                        // Images
                        else if (questionData["type"]["nomType"] === "image") {
                            for (const [j, choixData] of questionData["choix"].entries()) {
                                choix[j] = new Choix(choixData["idChoix"], choixData["image"], choixData["isCorrect"]);
                            }
                        }
                        const vf: boolean = questionData["type"]["nomType"] === "vraiFaux";
                        questions[i] = new Question(
                            questionData["question"],
                            questionData["idQuestion"],
                            questionData["nbPtsNegatif"],
                            questionData["nbPtsPositif"],
                            false,
                            false,
                            choix,
                            image,
                            vf,
                            false,
                            undefined,
                            undefined,
                            undefined,
                            questionData["position"]
                        );
                    }
                    // Numerique
                    else if (questionData["type"]["nomType"] === "numerique") {
                        questions[i] = new Question(
                            questionData["question"],
                            questionData["idQuestion"],
                            questionData["nbPtsNegatif"],
                            questionData["nbPtsPositif"],
                            false,
                            false,
                            undefined,
                            image,
                            false,
                            true,
                            0,
                            undefined,
                            undefined,
                            questionData["position"]
                        );
                    }
                }                
                const qcm : QcmData = new QcmData(
                    data["nomQCM"],
                    data["temps"],
                    data["randomOrder"],
                    questions,
                    data["temps"],
                    data["idQCM"],
                )
                resolve(qcm)
            })
            .catch(error => reject(error));
        });
    }
    static getQcm(QCM_ID: number) : Promise<QcmData>{
        return new Promise ((resolve, reject) => {
            this.requeteServeur({
                method: 'GET',
                address: Fetch.address,
                route: `examen_QCM/${QCM_ID}`,
            })
            .then(response => response.json())
            .then(data => {
                const questions: Question[] = []
                for (const [i, questionData] of data["qcm"]["questions"].entries()) {
                    const image: string = "";
                    if (["image", "text", "vraiFaux"].includes(questionData["type"]["nomType"])){
                        const choix: Choix[] = [];
                        //Textes
                        if(["text", "vraiFaux"].includes(questionData["type"]["nomType"])){
                            for (const [j, choixData] of questionData["choix"].entries()) {
                                choix[j] = new Choix(choixData["idChoix"], choixData["nomChoix"])
                            }  
                        }
                        //images
                        else if (questionData["type"] === "image"){
                            for (const [j, choixData] of questionData["choix"].entries()) {
                                choix[j] = new Choix(choixData["idChoix"], choixData["image"])
                            } 
                        }
                        questions[i] = new Question(questionData["question"],
                        questionData["idQuestion"], 
                        questionData["nbPtsNegatif"], 
                        questionData["nbPtsPositif"], 
                        questionData["isMultiple"],
                        false,
                        choix,
                        image,
                        false,
                        false
                    )
                    }
                    //Numerique
                    else if(questionData["type"]["nomType"] === "numerique"){
                        questions[i] = new Question(questionData["question"],
                            questionData["idQuestion"], 
                            questionData["nbPtsNegatif"], 
                            questionData["nbPtsPositif"], 
                            false,
                            false,
                            undefined,
                            image,
                            false,
                            true,
                            0
                        )
                    }
                }
                const qcm = new QcmData(
                    data["qcm"]["nomQCM"],
                    data["qcm"]["temps"],
                    false,
                    questions,
                    data["heureDebut"],
                    QCM_ID
                )
                resolve(qcm)
            })
            .catch(error => reject(error));
        });
    }

    // ----- Post -----
    static joinQcm(idQcm: number, codeAccess: number, USER_ID: number): Promise<Response> {
        return new Promise<Response>((resolve, reject) => {                
            fetch(Fetch.address+'/join/', {
                method: 'POST',
                headers: this.headersObject,
                body: JSON.stringify({
                    codeAcces: codeAccess,
                    idQCM: idQcm,
                    idUser: USER_ID
                })
            })
            .then(response => {
                resolve(response);
            })
            .catch(error => reject(error));
        });
    }
    static postQuestionText(question: Question, idQcm: number): Promise<number> {
        return new Promise<number>((resolve, reject) => {
            fetch(Fetch.address+'/text_question/', {
                method: 'POST',
                headers: this.headersObject,
                body: JSON.stringify({
                    question: question,
                    idQcm: idQcm,
                })
            })
            .then(response => response.json())

            .then(data => {
                if(data["error"]){
                    alert(data["error"]);
                    resolve(-1);
                }
                if(typeof data["id"] === 'number'){
                    resolve(data["id"]);
                }
            })
            .catch(error => reject(error));
        });
    }
    static postQuestionVraiFaux(question: Question, idQcm: number): Promise<number> {
        console.log(idQcm)
        return new Promise<number>((resolve, reject) => {
            fetch(Fetch.address+'/true_false_question/', {
                method: 'POST',
                headers: this.headersObject,
                body: JSON.stringify({
                    question: question,
                    idQcm: idQcm
                })
            })
            .then(response => response.json())
            .then(data => {
                if(typeof data["id"] === 'number'){
                    resolve(data["id"])
                }
            })
            .catch(error => reject(error));
        });
    }
    static postQuestionNumerique(question: Question, idQcm: number): Promise<number> {
        return new Promise<number>((resolve, reject) => {
            fetch(Fetch.address+'/numeric_question/', {
                method: 'POST',
                headers: this.headersObject,
                body: JSON.stringify({
                    question: question,
                    idQcm: idQcm,
                })
            })
            .then(response => response.json())
            .then(data => {
                if(typeof data["id"] === 'number'){
                    resolve(data["id"])
                }
            })
            .catch(error => reject(error));
        });
    }
    static postQcm(qcmData: QcmData, idUser: number): Promise<boolean> {
        return new Promise<boolean>((resolve, reject) => {
            console.log("on va essayer de creer : ")
            console.log(qcmData)
            fetch(Fetch.address+'/QCM', {
                method: 'POST',
                headers: this.headersObject,
                body: JSON.stringify({
                    qcm: qcmData,
                    idUser: idUser,
                })
            })
            .then(response => {
                console.log("response de la creation qcm:")
                console.log(response)
                if (!response.ok) {
                    reject(`HTTP error! status: ${response.status}`);
                }
                response.json().then(data => {qcmData.idQcm = data["id"]; resolve(true);})
                
            })
            .catch(error => reject(error));
        });
    }
    static postReponse(idChoix: number, idQuestion: number, USER_ID: number): Promise<number> {
        return new Promise<number>((resolve, reject) => {
            fetch(Fetch.address+'/response/', {
                method: 'POST',
                headers: this.headersObject,
                body: JSON.stringify({
                    idQuestion: idQuestion,
                    idChoix: idChoix,
                    idUser: USER_ID
                })
            })
            .then(response => {
                if (!response.ok) {
                    reject(`HTTP error! status: ${response.status}`);
                }
                response.json().then(data => resolve(data["idReponse"]))
            })
            .catch(error => reject(error));
        });
    }
    static postReponseNumerique(val: number, idQuestion: number, USER_ID: number): Promise<number> {
        return new Promise<number>((resolve, reject) => {
            fetch(Fetch.address+'/numeric_response/', {
                method: 'POST',
                headers:this.headersObject,
                body: JSON.stringify({
                    idQuestion: idQuestion,
                    answer: val,
                    idUser: USER_ID
                })
            })
            .then(response => {
                if (!response.ok) {
                    reject(`HTTP error! status: ${response.status}`);
                }
                response.json().then(data => resolve(data["idReponse"]))
            })
            .catch(error => reject(error));
        });
    }

    // ----- Put -----
    static putTerminer(idUser: number, qcmId: number): Promise<Response> {
        return new Promise<Response>((resolve, reject) => {
            fetch(Fetch.address+`/terminer/${idUser}/${qcmId}`, {
                method: 'PUT',
                headers: this.headersObject 
            })
            .then(response => {
                if (!response.ok) {
                    reject(`HTTP error! status: ${response.status}`);
                }
                resolve(response);
            })
            .catch(error => reject(error));
        });
    }
    static putQcm(qcmData: QcmData): Promise<Response> {
        return new Promise<Response>((resolve, reject) => {
            
            
            fetch(Fetch.address+'/QCM', {
                method: 'PUT',
                headers: this.headersObject,
                body: JSON.stringify({
                    qcm: qcmData,
                })
            })
            .then(response => {
                
                if (!response.ok) {
                    reject(`HTTP error! status: ${response.status}`);
                }
                resolve(response);
            })
            .catch(error => reject(error));
        });
    } 
    static putReponseChoix(idChoix: number, idQuestion: number, _USER_ID: number): Promise<Response> {
        return new Promise<Response>((resolve, reject) => {
            fetch(Fetch.address+'/reponseChoix/', {
                method: 'PUT',
                headers: this.headersObject,
                body: JSON.stringify({
                    idQuestion: idQuestion,
                    idChoix: idChoix,
                    idUser: _USER_ID
                })
            })
            .then(response => {
                if (!response.ok) {
                    reject(`HTTP error! status: ${response.status}`);
                }
                resolve(response);
            })
            .catch(error => reject(error));
        });
    }
    static putFeedBack(fd: FeedBack, idQcm: number, idUser: number): Promise<Response> {
        return new Promise<Response>((resolve, reject) => {
            console.log("asf")
            fetch(Fetch.address+'/feedBack/', {
                method: 'PUT',
                headers: this.headersObject,
                body: JSON.stringify({
                    idQCM: idQcm,
                    idUser: idUser,
                    feedback: fd.feedBack,
                    note: fd.note
                })
            })
            .then(response => {
                if (!response.ok) {
                    reject(`HTTP error! status: ${response.status}`);
                }
                resolve(response);
            })
            .catch(error => reject(error));
        });
    }

    // ----- Delete -----
    static deleteQuestion(idQuestion: number): Promise<Response> {
        return new Promise<Response>((resolve, reject) => {
            fetch(Fetch.address+`/question/${idQuestion}`, {
                method: 'DELETE',
                headers: Fetch.headersObject,
            })
            .then(response => {
                if (!response.ok) {
                    reject(`HTTP error! status: ${response.status}`);
                }
                resolve(response);
            })
            .catch(error => reject(error));
        });
    }
    static deleteReponseNumeric(idReponse: number, USER_ID: number): Promise<Response> {
        return new Promise<Response>((resolve, reject) => {
            fetch(Fetch.address+`/numeric_response/${idReponse}`, {
                method: 'DELETE',
                headers: Fetch.headersObject,
                body: JSON.stringify({
                    idUser: USER_ID
                })
            })
            .then(response => {
                if (!response.ok) {
                    reject(`HTTP error! status: ${response.status}`);
                }
                resolve(response);
            })
            .catch(error => reject(error));
        });
    }
    static deleteReponse(idResponse: number): Promise<Response> {
        return new Promise<Response>((resolve, reject) => {
            fetch(Fetch.address+`/response/${idResponse}`, {
                method: 'DELETE',
                headers: Fetch.headersObject,
            })
            .then(response => {
                if (!response.ok) {
                    reject(`HTTP error! status: ${response.status}`);
                }
                resolve(response);
            })
            .catch(error => reject(error));
        });
    }
}