import express         from 'express';

import db               from '../helpers/DatabaseHelper.js';
import { verifyJWT } from '../Middlewares.js';
import { MessageRoute } from './MessageRoute.js';
import { calcNbPtsTotalQCM,calcNoteBaremeFed,checkUser,calcTempsRestant} from "../calcFunctions.js"


const router: express.Router = express.Router();


router.delete('/response/:ANSWER_ID',verifyJWT, async (req: express.Request, res: express.Response) => {
    const ANSWER_ID: number = parseInt(req.params.ANSWER_ID, 10);
    if (!ANSWER_ID)
    {
        res.status(400).send({ error:  MessageRoute.misPara });
        return
    }
    const responseExist = await db.reponse.findFirst({
        where: {
            idReponse: ANSWER_ID
        },
    });

    console.log(responseExist);
    if (!responseExist)
    {
        res.status(404).send({ error: "This response doesn't exist and canno't be deleted" });
        return
    }
    checkUser(req, res, responseExist.idUser, "You can't delete a response in this QCM.");
    
    try {
        const reponse = await db.reponse.deleteMany({
            where: {
                idReponse: ANSWER_ID
            },
        });
        res.status(200).send(reponse);
    } catch (error) {
        res.status(500).send({ error:  MessageRoute.serverErrorAnswering });
    }
});

router.post('/response', verifyJWT, async (req: express.Request, res: express.Response) => {
    const { idQuestion, idChoix, idUser } = req.body;
    if (!idQuestion || !idChoix || !idUser)
    {
        res.status(400).send({ error: MessageRoute.misPara});
        return
    }
    checkUser(req, res, idUser, "You can't create a response in this QCM.");

    const infosQuestion = await db.choix.findFirst({
        where: {
            idQuestion: idQuestion
        },
        select: {
            idQuestion: true,
            idChoix: true
        }
    });

    if (infosQuestion)
    {
        const infosReponse = await db.reponse.findFirst({
            where: {
                AND: {
                    idQuestion: idQuestion,
                    idUser: idUser,
                    idChoix: idChoix
                }
            }
        });
        if (infosReponse?.idChoix)
        {
            res.status(200).send({ error: "This choice has already been chosen"});
            return
        }
    }
    else {
        res.status(404).send({ error: MessageRoute.questionDoesntExiste});
        return
    }
    
    try {
        const reponse = await db.reponse.create({
            data: {
                idQuestion: idQuestion,
                idChoix: idChoix,
                idUser: idUser,
            }
        });
        res.status(201).send(reponse);
    } catch (error) {
        res.status(500).send({ error:  MessageRoute.serverErrorAnswering });
    }
});

router.delete('/numeric_response/:ANSWER_ID', verifyJWT,async (req: express.Request, res: express.Response) => {
    const ANSWER_ID: number = parseInt(req.params.ANSWER_ID, 10);
    console.log(ANSWER_ID);
    const responseExist = await db.reponse.findFirst({
        where: {
            idReponse: ANSWER_ID
        }
    });

    console.log(responseExist);

    if (!responseExist)
    {
        res.status(404).send({ error: "This response doesn't exist and canno't be deleted" });
        return
    }
    checkUser(req, res, responseExist.idUser,  MessageRoute.serverErrorAnswering);
    
    try {
        const reponse = await db.reponse.delete({
            where: {
                idReponse: ANSWER_ID
            },
        });
        res.status(200).send(reponse);
    } catch (error) {
        res.status(500).send({ error: MessageRoute.serverErrorAnswering });
    }
});


router.put('/terminer/:USER_ID/:QCM_ID',verifyJWT, async (req: express.Request, res: express.Response) => {
    const USER_ID: number = parseInt(req.params.USER_ID, 10);
    const QCM_ID: number = parseInt(req.params.QCM_ID, 10);
    checkUser(req, res, USER_ID, "This is not your QCM");
    try {
        const participation = await db.participer.findUnique({
            where: {
                idUser_idQCM: {
                    idUser: USER_ID,
                    idQCM: QCM_ID
                }
            }
        });
        if (!participation) 
        {
            res.status(404).send({ error: "This user didn't begin the QCM" });
            return
        }
        const questions = await db.question.findMany({
            where: {
                idQCM: QCM_ID
            },
            include: {
                choix: true
            }
        });
        const responses = await db.reponse.findMany({
            where: {
                idUser: USER_ID,
                question: {
                    idQCM: QCM_ID
                }
            },
            include: {
                choix: true,
                question: {
                    include: {
                        choix: true
                    }
                }
            }
        });

        let nbPtsObtenus: number = 0;
        
        for (const question of questions) {
            const userResponses = responses.filter(response => response.idQuestion === question.idQuestion);
            const correctChoices = question.choix.filter(choice => choice.isCorrect);

            let isCorrect = true;
            if(question.numeric){
                if(userResponses.length === 1){
                    if (userResponses[0].numeric !== question.numeric ) 
                    {
                        isCorrect = false
                    }
                }
                else{
                    isCorrect = false
                }
            }
            else{
                if (userResponses.length === correctChoices.length) 
                {
                    for (const correctChoice of correctChoices) {
                        if (!userResponses.some(response => response.idChoix === correctChoice.idChoix)) {
                            isCorrect = false;
                            break;
                        }
                    }
                }
                else {
                    isCorrect = false;
                }
            }


            if (isCorrect) {
                nbPtsObtenus += question.nbPtsPositif;
            } 
            else {
                nbPtsObtenus -= question.nbPtsNegatif;
            }
        }
        if (nbPtsObtenus < 0)
        {
            nbPtsObtenus = 0;
        }
        const nbPtsTotal = await calcNbPtsTotalQCM(QCM_ID);
        const noteTmp: number = calcNoteBaremeFed(nbPtsObtenus, nbPtsTotal);
        const reponse = await db.participer.update({
            where: {
                idUser_idQCM: {
                    idUser: USER_ID,
                    idQCM: QCM_ID
                }
            },
            data: {
                hasFinished: true,
                note: noteTmp,
                score: nbPtsObtenus
            },
        });
        res.status(200).send(reponse);
    } catch (error) {
        res.status(500).send({ error: 'Server error while answering to the question' });
    }
});

router.post('/join',verifyJWT, async (req: express.Request, res: express.Response) => {
    const { codeAcces, idQCM, idUser } = req.body;
    if (!codeAcces || !idQCM || !idUser)
    {
        res.status(400).send({ error: MessageRoute.misPara});
        return
    } 
    try {
        const userAlreadyJoin = await db.participer.findUnique({
            where: {
                idUser_idQCM: {
                    idUser: idUser,
                    idQCM: idQCM
                }
            },
            select: {
                heureDebut: true,
                hasFinished: true
            }
        });

        const infosQCM = await db.qcmTable.findUnique({
            where: {
                idQCM: idQCM
            },
            select: {
                codeAcces: true,
                temps: true,
                questions: {
                    select :{
                        idQCM: true,
                    }
                }
            }
        });

        if (infosQCM)
        {
            console.table(infosQCM.questions)
            if(infosQCM.questions.length === 0){
                res.status(401).send({ error: "QCM don't have questions"});
                return
            }
            if (codeAcces !== infosQCM.codeAcces)
            {
                res.status(401).send({ error: "Wrong access code"});
                return
            }
        }
        else {
            res.status(404).send({ error: "QCM doesn't exist"});
            return
        }

        // L'utilisateur à déjà rejoint le QCM
        if (userAlreadyJoin)
        {
            // Vérifie le temps restant
            if (await canAccessQCM(userAlreadyJoin.heureDebut, infosQCM.temps, userAlreadyJoin.hasFinished, idUser, idQCM))
            {
                res.status(200).send({ hasParticipated: true, hasTime: true });
                return
            }
            else {
                res.status(200).send({ hasParticipated: true, hasTime: false });
                return
            }
        }
        const heureDebut: number = Math.floor(Date.now() / 1000);

        const participation = await db.participer.create({
            data: {
                idUser: idUser,
                idQCM: idQCM,
                heureDebut: heureDebut,
                feedback: "",
                note: -1,
                score: 0
            }
        });
        res.status(201).send(participation);
        return
    } catch (error) {
        res.status(500).send({ error: 'Server error while joining QCM' });
        return
    }
});

async function canAccessQCM(heureDebut: number, tempsQCM: number, _hasFinished: boolean, idUser: number, idQCM: number)
{
    const tempsRestant = calcTempsRestant(heureDebut, tempsQCM);
    console.log(tempsRestant);
    if (tempsRestant <= 0)
    {
        await db.participer.update({
            where: {
                idUser_idQCM: {
                    idUser: idUser,
                    idQCM: idQCM
                }
            },
            data: {
                hasFinished: true,
            },
        });
        return false;
    }
    else {
        const hasFinished = await db.participer.findUnique({
            where: {
                idUser_idQCM: {
                    idUser: idUser,
                    idQCM: idQCM
                }
            },
            select: {
                hasFinished: true,
            }
        });
        console.table(hasFinished?.hasFinished);
        return !hasFinished?.hasFinished;
    }
}
router.delete('/question/:QUESTION_ID', verifyJWT, async (req: express.Request, res: express.Response) => {
    const QUESTION_ID: number = parseInt(req.params.QUESTION_ID, 10);
    console.log(QUESTION_ID);
    const questionExist = await db.question.findFirst({
        where: {
            idQuestion: QUESTION_ID
        },
        include: {
            qcm: true
        }
    });

    if (!questionExist)
    {
        res.status(404).send({ error: "This response doesn't exist and canno't be deleted" });
        return
    }
    checkUser(req, res, questionExist.qcm.idUserCreator, "You can't delete a question in this QCM.");

    try {
        console.log(QUESTION_ID)
        const reponse = await db.question.delete({
            where: {
                idQuestion: QUESTION_ID
            },
        });
        res.status(200).send(reponse);
    } catch (error) {
        console.error(error);
        res.status(500).send({ error: 'Server error while answering to the question' });
    }
});


router.post('/text_question', verifyJWT, async (req: express.Request, res: express.Response) => {
    const { question, idQcm } = req.body;
    console.log(question, idQcm);
    if (!idQcm || !question || !question["question"] || question["nbPtsNegatif"] === undefined  || question["nbPtsPositif"] === undefined || question["isMultiple"] === undefined || question["position"] === undefined || question["choix"] === undefined || question["randomOrder"] === undefined)
    {
        res.status(400).send({ error: MessageRoute.misPara });
        return
    }
    let minOneCorrectChoice: boolean = false;
    let minOneFalseChoice: boolean = false;
    
    for (const choix of question["choix"]) {
        if (choix["isCorrect"]) {
            minOneCorrectChoice = true;
        }
        if (!choix["isCorrect"]) {
            minOneFalseChoice = true;
        }
    }
    if (!minOneCorrectChoice)
    {
        res.status(409).send({ error: 'Missing a correct choice' });
        return
    }
    if (!minOneFalseChoice)
    {
        res.status(409).send({ error: 'Missing a false choice' });
        return
    }
    const infoQcm = await db.qcmTable.findFirst({
        where: {
            idQCM: idQcm,
        }
    });
    if(!infoQcm){
        res.status(404).send({ error: MessageRoute.qcmDoesntExist });
        return
    }
    checkUser(req, res, infoQcm.idUserCreator, MessageRoute.cantCreate);
    const type = await db.type.findFirst({
        where: {
            nomType: "text"
        },
    });
    if(!type){
        res.status(500).send({ error: MessageRoute.serverError });
        return
    }
    const questionCreate = await db.question.create({
        data: {
            nbPtsNegatif:question["nbPtsNegatif"],
            nbPtsPositif: question["nbPtsPositif"],
            position: question["position"],
            isMultiple: question["isMultiple"],
            question: question["question"],  
            idQCM: idQcm,
            idType: type["idType"],
            randomOrder: question["randomOrder"],
        }
    });
    console.log(questionCreate)
    if(!questionCreate){
        res.status(500).send({ error:  MessageRoute.serverError});
        return
    }
    const idQuestion = questionCreate["idQuestion"];
    for(let i = 0; i < question["choix"].length; i++){
        if(!question["choix"][i]["text"] || question["choix"][i]["isCorrect"] === undefined){
            res.status(500).send({ error: 'Server error' });
            return
        }
        const choixCreate = await db.choix.create({
            data: {
                nomChoix: question["choix"][i]["text"],
                isCorrect: question["choix"][i]["isCorrect"],
                idQuestion: idQuestion,
                position: i,
            }
        });
        if(!choixCreate){
            res.status(500).send({ error:  MessageRoute.serverError });
            return
        }
    }
    res.status(200).send({id: idQuestion});
});
router.post('/numeric_response',verifyJWT,  async (req: express.Request, res: express.Response) => {
    const { idQuestion, answer, idUser } = req.body;
    if (!idQuestion || !answer || !idUser)
    {
        res.status(400).send({ error:  MessageRoute.misPara });
        return
    }
    checkUser(req, res, idUser, "You can't create a response in this QCM.");

    const infosQuestion = await db.question.findFirst({
        where: {
            idQuestion: idQuestion
        }
    });

    if (infosQuestion)
    {
        const infosReponse = await db.reponse.findFirst({
            where: {
                AND: {
                    idQuestion: idQuestion,
                    idUser: idUser,
                    numeric: {
                        not: null
                    }
                }
            }
        });
        if (infosReponse)
        {
            res.status(200).send({ error: "User already answered this question" });
            return
        }
    }
    else {
        res.status(404).send({ error: MessageRoute.questionDoesntExiste});
        return
    }
    
    try {
        const reponse = await db.reponse.create({
            data: {
                idQuestion: idQuestion,
                idUser: idUser,
                numeric: answer
            }
        });
        res.status(201).send(reponse);
    } catch (error) {
        res.status(500).send({ error:  MessageRoute.serverErrorAnswering });
    }
});


export default router;
